@@ -15,10 +15,9 @@ def main(hub, args):
     if venv:
         vpath = py.path.local(venv)
         if not vpath.check():
-            hub.popen_check(["virtualenv", venv] + xopt)
+            hub.popen_check(["virtualenv", "-q", venv] + xopt)
     pip_path = current.getvenvbin("pip", venvdir=venv, glob=True)
     if not pip_path:
-        import pdb ; pdb.set_trace()
         hub.fatal("no pip binary found")
     if args.listinstalled:
         hub.info("list of installed packages on", current.venvdir)
@@ -35,6 +34,8 @@ def main(hub, args):
         except KeyError:
             pass
         hub.popen_check([pip_path, "install"] + xopt + [
+            #"--use-wheel",
+            "--pre",
             "-U", #"--force-reinstall",
             "-i", current.simpleindex ] + list(args.pkgspecs))
 
