@@ -4,19 +4,24 @@ import sys
 import py
 import argparse
 import subprocess
+import devpi
 from devpi.util.lazydecorator import lazydecorator
 from devpi.util import url as urlutil
 from devpi import log, cached_property
 from devpi.use import Current
-import devpi.server
 import requests
 import json
-from devpi.server import handle_autoserver
 std = py.std
 subcommand = lazydecorator()
 
 from devpi import __version__
 
+main_description = """
+The devpi commands (installed via devpi-client) wrap common Python
+packaging, uploading, installation and testing activities, using a remote
+devpi-server managed index.  For more information see http://doc.devpi.net
+"""
+
 def main(argv=None):
     if argv is None:
         argv = list(sys.argv)
@@ -48,6 +53,8 @@ def check_output(*args, **kwargs):
             raise CalledProcessError(retcode, cmd, output=output)
     return output
 
+notset = object()
+
 class Hub:
     class Popen(std.subprocess.Popen):
         STDOUT = std.subprocess.STDOUT
@@ -61,6 +68,8 @@ class Hub:
         self.args = args
         self.cwd = py.path.local()
         self.quiet = False
+        self._last_http_status = None
+        self._session = requests.session()
 
     def set_quiet(self):
         self.quiet = True
@@ -70,98 +79,77 @@ class Hub:
         return py.path.local(self.args.clientdir)
 
     def require_valid_current_with_index(self):
-        if not self.current.simpleindex:
-            handle_autoserver(self, self.current)
-        return self.current
-
+        current = self.current
+        if not current.index:
+            self.fatal("not connected to an index, see 'devpi use'")
+        return current
 
     # remote http hooks
 
-    @cached_property
+    @property
     def http(self):
-        session = requests.session()
-        p = self.clientdir.join("login")
-        if p.check():
-            data = json.loads(p.read())
-            session.auth = data["user"], data["password"]
+        session = self._session
+        #session.auth = tuple(self.current.auth) if self.current.auth else None
         session.ConnectionError = requests.exceptions.ConnectionError
         return session
 
-    def http_api(self, method, url, kvdict=None, quiet=False):
-        methodexec = getattr(self.http, method, None)
+    def http_api(self, method, url, kvdict=None, quiet=False, auth=notset):
+        """ send a json request and return a HTTPReply object which
+        adds some extra methods to the requests's Reply object.
+
+        This method will bail out if we could not connect or
+        the response code is >= 400.
+
+        This method will not output any information to the user
+        if ``quiet`` is set to true.
+        """
         jsontype = "application/json"
         headers = {"Accept": jsontype, "content-type": jsontype}
-        if method in ("delete", "get"):
-            r = methodexec(url, headers=headers)
-        elif method == "push":
-            r = self.http.request(method, url, data=json.dumps(kvdict),
-                                  headers=headers)
-        else:
-            r = methodexec(url, json.dumps(kvdict), headers=headers)
-        if r.status_code < 0:
-            self.fatal("%s: could not connect to %r" % (r.status_code, url))
-        out = self.info
+        try:
+            data = json.dumps(kvdict) if kvdict is not None else None
+            if auth is notset:
+                auth = self.current.get_auth()
+            r = self.http.request(method, url, data=data, headers=headers,
+                                  auth=auth)
+        except self.http.ConnectionError:
+            self._last_http_status = -1
+            self.fatal("could not connect to %r" % (url,))
+        self._last_http_status = r.status_code
+
+        reply = HTTPReply(r)
+
+        # if we get a 401 it means our auth info is expired or not present
+        if r.status_code == 401:
+            if self.current.del_auth():
+                self.error("removed expired authentication information")
+
+        if r.status_code in (200, 201):
+            # don't show any extra info on success code
+            return reply
+        # feedback reply info to user, possibly bailing out
         if r.status_code >= 400:
             out = self.fatal
-        elif quiet:
-            out = lambda *args, **kwargs: None
+        elif quiet and not self.args.debug:
+            return reply
+        else:
+            out = self.info
 
         if r.status_code >= 400 or self.args.debug:
-            info = "%s %s\n" % (method.upper(), r.url)
+            info = "%s %s\n" % (r.request.method, r.url)
         else:
             info = ""
-        data = r.content
-        if data and r.headers["content-type"] == "application/json":
-            data = json.loads(data)
-            reason = data.get("message", r.reason)
-        else:
-            reason = r.reason
-        out("%s%s: %s" %(info, r.status_code, reason))
-        return data
+        message = reply.json_get("message", "")
+        if message:
+            message = ": " + message
+        out("%s%s %s%s" %(info, r.status_code, r.reason, message))
+        return reply
 
-    def update_auth(self, user, password):
-        self.http.auth = (user, password)
-        self.clientdir.ensure(dir=1)
-        oldumask = os.umask(7*8+7)
-        try:
-            self.clientdir.join("login").write(
-                json.dumps(dict(user=user, password=password)))
-        finally:
-            os.umask(oldumask)
 
-    def delete_auth(self):
-        loginpath = self.clientdir.join("login")
-        if loginpath.check():
-            loginpath.remove()
 
     def requires_login(self):
-        if not self.http.auth:
+        if not self.current.get_auth_user():
             self.fatal("you need to be logged in (use 'devpi login USER')")
 
-    def get_index_url(self, indexname=None, current=None, slash=True):
-        if current is None:
-            current = self.current
-        if indexname is None:
-            indexname = current.index
-            if indexname is None:
-                raise ValueError("no index name")
-        if "/" not in indexname:
-            assert self.http.auth[0]
-            userurl = current.getuserurl(self.http.auth[0])
-            return urlutil.joinpath(userurl + "/", indexname)
-        url = urlutil.joinpath(current.rooturl, indexname)
-        if slash:
-            url = url.rstrip("/") + "/"
-        return url
-
-    def get_project_url(self, name):
-        baseurl = self.get_index_url(slash=True)
-        url = urlutil.joinpath(baseurl, name) + "/"
-        return url
-
-    def get_user_url(self):
-        return self.current.getuserurl(self.http.auth[0])
-
     def raw_input(self, msg):
         return raw_input(msg)
 
@@ -184,8 +172,6 @@ class Hub:
         current = Current(path)
         try:
             cmd = self.args.mainloc.split(":")[0]
-            if cmd not in ("devpi.use", "devpi.server"):
-                handle_autoserver(self, current)
         except AttributeError:
             pass
         return current
@@ -218,6 +204,22 @@ class Hub:
             return
         return check_output(args, cwd=str(cwd))
 
+    def popen(self, args, cwd=None):
+        if isinstance(args, str):
+            args = std.shlex.split(args)
+        assert args[0], args
+        args = [str(x) for x in args]
+        if cwd == None:
+            cwd = self.cwd
+        self.report_popen(args, cwd)
+        if self.args.dryrun:
+            return
+        popen = subprocess.Popen(args, cwd=str(cwd))
+        out, err = popen.communicate()
+        ret = popen.wait()
+        if ret:
+            self.fatal("****** process returned %s" % ret)
+
     def report_popen(self, args, cwd=None):
         base = cwd or self.cwd
         rel = py.path.local(args[0]).relto(base)
@@ -274,6 +276,30 @@ class Hub:
         self._tw.line(json.dumps(data, sort_keys=True, indent=4))
 
 
+class HTTPReply(object):
+    def __init__(self, response):
+        self._response = response
+
+    def __getattr__(self, name):
+        return getattr(self._response, name)
+
+    def json_get(self, jsonkey, default=None):
+        r = self._response
+        if not r.content or r.headers["content-type"] != "application/json":
+            return default
+        try:
+            return self._json[jsonkey]
+        except KeyError:
+            return default
+
+    @cached_property
+    def _json(self):
+        return self._response.json()
+
+    def __getitem__(self, name):
+        return self._json[name]
+
+
 class MyArgumentParser(argparse.ArgumentParser):
     class ArgumentError(Exception):
         """ and error from the argparse subsystem. """
@@ -281,6 +307,13 @@ class MyArgumentParser(argparse.ArgumentParser):
         """raise errors instead of printing and raising SystemExit"""
         raise self.ArgumentError(error)
 
+    #def __init__(self, *args, **kwargs):
+    #    kwargs["formatter_class"] = MyHelpFormatter
+    #    argparse.ArgumentParser.__init__(self, *args, **kwargs)
+
+#class MyHelpFormatter(argparse.HelpFormatter):
+#    pass
+
 def try_argcomplete(parser):
     try:
         import argcomplete
@@ -291,7 +324,7 @@ def try_argcomplete(parser):
 
 def parse_args(argv):
     argv = map(str, argv)
-    parser = getbasebaser(argv[0])
+    parser = getbaseparser(argv[0])
     add_subparsers(parser)
     try_argcomplete(parser)
     try:
@@ -302,6 +335,15 @@ def parse_args(argv):
         parser.print_usage()
         parser.exit(2, "%s: error: %s\n" % (parser.prog, e.args[0]))
 
+def parse_docstring(txt):
+    description = txt
+    i = txt.find(".")
+    if i == -1:
+        doc = txt
+    else:
+        doc = txt[:i+1]
+    return doc, description
+
 def add_subparsers(parser):
     subparsers = parser.add_subparsers()
     for func, args, kwargs in subcommand.discover(globals()):
@@ -309,7 +351,10 @@ def add_subparsers(parser):
             name = args[1]
         else:
             name = func.__name__
-        subparser = subparsers.add_parser(name, help=func.__doc__)
+        doc, description = parse_docstring(func.__doc__)
+        subparser = subparsers.add_parser(name,
+                                          description=description,
+                                          help=doc)
         subparser.Action = argparse.Action
         add_generic_options(subparser)
         func(subparser)
@@ -318,8 +363,8 @@ def add_subparsers(parser):
     #subparser = subparsers.add_parser("_test", help=argparse.SUPPRESS)
     #subparser.set_defaults(mainloc="devpi")
 
-def getbasebaser(prog):
-    parser = MyArgumentParser(prog=prog)
+def getbaseparser(prog):
+    parser = MyArgumentParser(prog=prog, description=main_description)
     add_generic_options(parser)
     return parser
 
@@ -338,18 +383,43 @@ def add_generic_options(parser):
                                                   "~/.devpi/client")),
         help="directory for storing login and other state")
 
+@subcommand("devpi.quickstart")
+def quickstart(parser):
+    """ start a server, create a user and login, then create a USER/dev
+    index and then connect to this index, so that subsequent devpi
+    commands can work with it.
+    """
+    parser.add_argument("--user", action="store",
+        default=os.environ.get("USER", "test"),
+        help="set initial user name to create and login")
+    parser.add_argument("--password", action="store",
+        default="",
+        help="initial password (default is empty)")
+    parser.add_argument("--index", action="store",
+        default="dev",
+        help="initial index name for the user.")
+    parser.add_argument("--dry-run", action="store_true", dest="dryrun",
+        default=False,
+        help="don't perform any actions, just show them")
+
 @subcommand("devpi.use")
 def use(parser):
     """ show/configure current index and target venv for install
-    activities. """
+    activities.
+
+    This shows client-side state, relevant for server interactions,
+    including login authentication information, the current remote index
+    (and API endpoints if you specify --urls) and the target virtualenv
+    for installation activities.
+    """
 
     parser.add_argument("--venv", action="store", default=None,
         help="set virtual environment to use for install activities. "
              "specify '-' to unset it.")
-    parser.add_argument("--no-auto", action="store_true", dest="noauto",
-        help="don't start automatic server")
     parser.add_argument("--urls", action="store_true",
         help="show remote endpoint urls")
+    parser.add_argument("-l", action="store_true", dest="list",
+        help="show all available indexes at the remote server")
     parser.add_argument("--delete", action="store_true",
         help="delete current association with server")
     parser.add_argument("url", nargs="?",
@@ -361,14 +431,27 @@ def use(parser):
 
 @subcommand("devpi.getjson")
 def getjson(parser):
-    """ show remote server and index configuration. """
-    parser.add_argument("path", nargs="?",
+    """ show remote server and index configuration.
+
+    A low-level command to show json-formatted configuration data
+    from remote resources.  This will always query the remote server.
+    """
+    parser.add_argument("path", action="store",
         help="path to a resource to show information on. "
              "examples: '/', '/user', '/user/index'.")
 
 @subcommand("devpi.list_remove:main_list", "list")
 def list_(parser):
-    """ list project versions and files for the current index"""
+    """ list project versions and files for the current index.
+
+    Without a ``spec`` argument this command will show the names
+    of all projects that have been used.  With a spec argument
+    it will show all release files.  RED files come from an
+    an inherited version which is shadowed by an inheriting index.
+    """
+    parser.add_argument("-f", "--failures", action="store_true",
+        dest="failures",
+        help="show test setup/failure logs")
     parser.add_argument("spec", nargs="?",
         help="show only info for a project/version/release file.  "
              "Example specs: 'pytest' or 'pytest-2.3.5' or "
@@ -376,7 +459,12 @@ def list_(parser):
 
 @subcommand("devpi.list_remove:main_remove")
 def remove(parser):
-    """ remove project info/files from current index. """
+    """ remove project info/files from current index.
+
+    This command allows to remove projects or releases from your
+    current index (see "devpi use").  It will ask interactively
+    for confirmation before performing the actual removals.
+    """
     parser.add_argument("spec",
         help="remove info/files for a project/version/release file from the "
              "current index. "
@@ -384,24 +472,36 @@ def remove(parser):
 
 @subcommand("devpi.user")
 def user(parser):
-    """ add, remove, modify, list user configuration"""
-    group = parser.add_argument_group()
-    group.add_argument("-c", "--create", action="store_true",
+    """ add, remove, modify, list user configuration.
+
+    This is the central command for performing remote user
+    configuration and manipulation.  Each indexes (created in turn
+    by "devpi index" command) is owned by a particular user.
+    If you create a user you either need to pass a ``password=...``
+    setting or interactively type a password.
+    """
+    parser.add_argument("-c", "--create", action="store_true",
         help="create a user")
-    group.add_argument("--delete", action="store_true",
+    parser.add_argument("--delete", action="store_true",
         help="delete a user")
-    group.add_argument("-m", "--modify", action="store_true",
+    parser.add_argument("-m", "--modify", action="store_true",
         help="modify user settings")
-    group.add_argument("-l", "--list", action="store_true",
+    parser.add_argument("-l", "--list", action="store_true",
         help="list user names")
     parser.add_argument("username", type=str, action="store", nargs="?",
         help="user name")
     parser.add_argument("keyvalues", nargs="*", type=str,
-        help="key=value configuration item")
+        help="key=value configuration item.  Possible keys: "
+             "email, password.")
 
 @subcommand("devpi.login")
 def login(parser):
-    """ login to devpi-server"""
+    """ login to devpi-server with the specified user.
+
+    This command performs the login protocol with the remove server
+    which typically results in a cached auth token which is valid for
+    ten hours.  You can check your login information with "devpi use".
+    """
     parser.add_argument("--password", action="store", default=None,
                         help="password to use for login (prompt if not set)")
     parser.add_argument("username", action="store", default=None,
@@ -409,28 +509,45 @@ def login(parser):
 
 @subcommand("devpi.login:logoff")
 def logoff(parser):
-    """ log out of the current devpi-server"""
+    """ log out of the current devpi-server.
+
+    This will erase the client-side login token (see "devpi login").
+    """
 
 @subcommand("devpi.index")
 def index(parser):
-    """ create, delete and manage indexes. """
+    """ create, delete and manage indexes.
+
+    This is the central command to create and manipulate indexes.
+    The index is always created under the currently logged in user
+    with a command like this: ``devpi index -c newindex``.
+
+    You can also view the configuration of any index with
+    ``devpi index USER/INDEX`` or list all indexes of the
+    in-use index with ``devpi index -l``.
+    """
     group = parser.add_mutually_exclusive_group()
     group.add_argument("-c", "--create", action="store_true",
         help="create an index")
     group.add_argument("--delete", action="store_true",
         help="delete an index")
-    group.add_argument("-m", "--modify", action="store_true",
-        help="modify an index")
     group.add_argument("-l", "--list", action="store_true",
         help="list indexes for the logged in user")
     parser.add_argument("indexname", type=str, action="store", nargs="?",
-        help="index name, specified as NAME or USER/NAME")
+        help="index name, specified as NAME or USER/NAME.  If no index "
+             "is specified use the current index")
     parser.add_argument("keyvalues", nargs="*", type=str,
-        help="key=value configuration item")
+        help="key=value configuration item. Possible key=value are "
+             "bases=CSV, volatile=True|False, acl_upload=CSV)")
 
 @subcommand("devpi.upload.upload")
 def upload(parser):
-    """ prepare and upload packages to the current index. """
+    """ prepare and upload packages to the current index.
+
+    This command wraps ``setup.py`` invocations to build and
+    upload releases, release files and documentation to your
+    in-use index (see "devpi use").
+    """
     #parser.add_argument("-l", dest="showstatus",
     #    action="store_true", default=None,
     #    help="show remote versions, local version and package types")
@@ -441,7 +558,7 @@ def upload(parser):
     #    help="retrieve max remove version, increment and set it like --ver")
     parser.add_argument("--formats", default="sdist.tgz", action="store",
         help="comma separated list of build formats (passed to setup.py). "
-             "Examples sdist.zip,bdist_egg,bdist_dumb.")
+             "Examples sdist.zip,bdist_egg,bdist_wheel,bdist_dumb.")
     parser.add_argument("--from-dir", action="store", default=None,
         dest="fromdir",
         help="upload all archive files from the specified directory")
@@ -453,18 +570,23 @@ def upload(parser):
         help="don't perform any server-modifying actions")
     parser.add_argument("--with-docs", action="store_true", default=None,
         dest="withdocs",
-        help="perform upload_docs in addition to uploading release files")
+        help="build sphinx docs and upload them to index. "
+             "this triggers 'setup.py build_sphinx ... upload_docs ...'")
     parser.add_argument("--only-docs", action="store_true", default=None,
         dest="onlydocs",
-        help="perform only upload_docs and no release files")
+        help="as --with-docs but don't upload release files")
     #parser.add_argument("-y", dest="yes",
     #    action="store_true", default=None,
     #    help="answer yes on interactive questions. ")
     #
 
-@subcommand("devpi.test.test")
+@subcommand("devpi.test")
 def test(parser):
-    """ download and test a package against tox environments."""
+    """ download and test a package against tox environments.
+
+    Download a package and run tests as configured by the
+    tox.ini file (which must be contained in the package).
+    """
     parser.add_argument("-e", metavar="VENV", type=str, dest="venv",
         default=None, action="store",
         help="virtual environment to run from the tox.ini")
@@ -475,8 +597,12 @@ def test(parser):
 
 @subcommand("devpi.push")
 def push(parser):
-    """ push a release and releasefiles to an external index server.
-        (pushing between indexes not implemented yet).
+    """ push a release and releasefiles to an internal or external index.
+
+        You can push a release with all its release files either
+        to an external pypi server ("pypi:REPONAME") where REPONAME
+        needs to be defined in your ``.pypirc`` file.  Or you can
+        push to another devpi index ("user/name").
     """
     parser.add_argument("--pypirc", metavar="path", type=str,
         default=None, action="store",
@@ -486,16 +612,21 @@ def push(parser):
         help="release in format 'name-version'. of which the metadata and "
              "all release files are to be uploaded to the specified "
              "external pypi repo." )
-    parser.add_argument("-r", dest="posturl", metavar="url", type=str,
-        default=None, action="store",
-        help="repo name as specified in your .pypirc file")
-    #parser.add_argument("targetindex", type=str, default=None, nargs="?",
-    #    help="index in USER/NAME form to push to. ")
+    parser.add_argument("target", metavar="TARGETSPEC", type=str,
+        action="store",
+        help="local or remote target index. local targets are of form "
+             "'USER/NAME', specifying an existing writeable local index. "
+             "remote targets are of form 'REPO:' where REPO must be an "
+             "existing entry in the pypirc file.")
 
 
 @subcommand("devpi.install")
 def install(parser):
-    """ install packages through current devpi index. """
+    """ install packages through current devpi index.
+
+    This is convenience wrapper which configures and invokes
+    ``pip install`` commands for you, using the current index.
+    """
     parser.add_argument("-l", action="store_true", dest="listinstalled",
         help="print list of currently installed packages. ")
     parser.add_argument("-e", action="store", dest="editable", metavar="ARG",
@@ -508,14 +639,6 @@ def install(parser):
         help="uri or package file for installation from current index. """
     )
 
-@subcommand("devpi.server")
-def server(parser):
-    """ commands for controling the automatic server. """
-    parser.add_argument("--stop", action="store_true",
-        help="stop automatically started server if any")
-    parser.add_argument("--log", action="store_true",
-        help="show last log lines of automatic server")
-
 
 if __name__ == "__main__":
     main()
