@@ -1,15 +1,16 @@
 
 # content of conftest.py
 
+import random
 import pytest
 import textwrap
 import py
 import sys
 import os
+import json
 
 from _pytest.pytester import RunResult, LineMatcher
 from devpi.main import Hub, initmain, parse_args
-from devpi.server import AutoServer
 import subprocess
 
 print_ = py.builtin.print_
@@ -19,10 +20,6 @@ pytest_plugins = "pytester"
 
 from devpi.util import url as urlutil
 
-@pytest.fixture(autouse=True, scope="session")
-def noautoserver():
-    std.os.environ["DEVPI_NO_AUTOSERVER"] = "1"
-
 def pytest_runtest_makereport(item, call):
     if "incremental" in item.keywords:
         if call.excinfo is not None:
@@ -85,7 +82,7 @@ def Popen(request):
 
 def get_pypirc_patcher(devpi):
     hub = devpi("use")
-    user, password = hub.http.auth
+    user, password = hub.current.get_auth()
     pypisubmit = hub.current.pypisubmit
     class overwrite:
         def __enter__(self):
@@ -118,20 +115,22 @@ def get_pypirc_patcher(devpi):
 
 @pytest.fixture(scope="session")
 def port_of_liveserver(request):
-    port = 7999
     if request.config.option.fast:
         pytest.skip("not running functional tests in --fast mode")
+    port = random.randint(2001, 64000)
     clientdir = request.config._tmpdirhandler.mktemp("liveserver")
-    hub, method = initmain(["devpi", "--clientdir", clientdir, "server"])
-    autoserver = AutoServer(hub)
-    autoserver.start("http://localhost:%s" % port, removedata=True)
-    request.addfinalizer(autoserver.stop)
+    subprocess.check_call(["devpi-server", "--serverdir", str(clientdir),
+                             "--port", str(port), "--start"])
+    def stop():
+        subprocess.check_call(["devpi-server", "--serverdir", str(clientdir),
+                                 "--stop"])
+    request.addfinalizer(stop)
     return port
 
 @pytest.fixture
 def devpi(cmd_devpi, gen, port_of_liveserver):
     user = gen.user()
-    cmd_devpi("use", "http://localhost:%s/root/dev" % port_of_liveserver)
+    cmd_devpi("use", "http://localhost:%s/root/pypi" % port_of_liveserver)
     cmd_devpi("user", "-c", user, "password=123", "email=123")
     cmd_devpi("login", user, "--password", "123")
     cmd_devpi("index", "-c", "dev")
@@ -294,9 +293,14 @@ def cmd_devpi(tmpdir):
             ret = method(hub, hub.args)
         except SystemExit as sysex:
             ret = sysex.args[0] or 1
-        if ret and kwargs.get("code", 0) < 400:
-            raise SystemExit(ret)
-        hub, method = initmain(callargs)
+            hub.sysex = sysex
+        expected = kwargs.get("code", None)
+        if expected is not None:
+            if not isinstance(expected, tuple):
+                expected = (expected, )
+            if hub._last_http_status not in expected:
+                pytest.fail("got http code %r, expected %r"
+                            % (hub._last_http_status, expected))
         return hub
     run_devpi.clientdir = clientdir
     return run_devpi
@@ -321,6 +325,7 @@ def runprocess(tmpdir, cmdargs):
 
 
 import pytest
+from mock import Mock
 import devpi.util.url as urlutil
 from devpi import log
 
@@ -367,7 +372,7 @@ def create_venv(request, testdir, monkeypatch):
 
 
 @pytest.fixture
-def loghub(tmpdir):
+def loghub(tmpdir, mock_http_api):
     class args:
         debug = True
         clientdir = tmpdir.join("clientdir")
@@ -385,10 +390,49 @@ def makehub(request):
     handler = request.config._tmpdirhandler
     def mkhub(arglist):
         tmp = handler.mktemp("hub")
-        args = parse_args(["testhub"] + arglist)
+        for x in arglist:
+            if "--clientdir" in x:
+                break
+        else:
+            arglist.append("--clientdir=%s" % tmp)
+        args = parse_args(["devpi_"] + arglist)
         old = tmp.chdir()
         try:
             return Hub(args)
         finally:
             old.chdir()
     return mkhub
+
+@pytest.fixture
+def mock_http_api(monkeypatch):
+    """ mock out all Hub.http_api calls and return an object
+    offering 'register_result' to fake replies. """
+    from devpi import main
+    #monkeypatch.replace("requests.session.Session.request", None)
+    from requests.sessions import Session
+    monkeypatch.setattr(Session, "request", None)
+
+    class MockHTTPAPI:
+        def __init__(self):
+            self.called = []
+            self._json_responses = {}
+
+        def __call__(self, method, url, kvdict=None, quiet=False, auth=None):
+            self.called.append((method, url, kvdict))
+            reply_data = self._json_responses.get(url)
+            if reply_data is not None:
+                class R:
+                    status_code = reply_data["status"]
+                    reason = reply_data.get("reason", "OK")
+                    def json(self):
+                        return reply_data["json"]
+                return main.HTTPReply(R())
+            pytest.fail("http_api call to %r is not mocked" % (url,))
+
+        def set(self, url, status=200, **kw):
+            data = json.loads(json.dumps(kw))
+            self._json_responses[url] = {"status": status, "json": data}
+    mockapi = MockHTTPAPI()
+    monkeypatch.setattr(main.Hub, "http_api", mockapi)
+    return mockapi
+
