@@ -26,10 +26,22 @@ def uploadhub(request, tmpdir):
                            "upload"])
     return hub
 
+#@pytest.fixture(autouse=True)
+#def extpypi_404(monkeypatch):
+#    from devpi_server import extpypi  # this is only client side
+#    monkeypatch.setattr(extpypi.ExtDB, "getreleaselinks", lambda *args: 404)
+
 
 class TestCheckout:
+    @pytest.fixture(autouse=True)
+    def no_sys_executable(self, monkeypatch):
+        """ make sure sys.executable is not used accidentally. """
+        monkeypatch.setattr(sys, "executable", None)
+
     @pytest.fixture(scope="class")
     def repo(self, request):
+        if not py.path.local.sysfind("hg"):
+            pytest.skip("'hg' command not found")
         repo = request.config._tmpdirhandler.mktemp("repo", numbered=True)
         file = repo.join("file")
         file.write("hello")
@@ -44,7 +56,6 @@ class TestCheckout:
         return repo
 
     def test_hg_export(self, uploadhub, repo, tmpdir, monkeypatch):
-
         checkout = Checkout(uploadhub, repo)
         assert checkout.rootpath == repo
         newrepo = tmpdir.mkdir("newrepo")
@@ -52,6 +63,20 @@ class TestCheckout:
         assert result.rootpath.join("file").check()
         assert result.rootpath == newrepo.join(repo.basename)
 
+    def test_hg_export_verify_setup(self, uploadhub, repo,
+                                          tmpdir, monkeypatch):
+        subdir = repo.mkdir("subdir")
+        subdir.ensure("setup.py")
+        checkout = Checkout(uploadhub, subdir)
+        wc = tmpdir.mkdir("wc")
+        exported = checkout.export(wc)
+        with pytest.raises(SystemExit):
+            exported.check_setup()
+        with pytest.raises(SystemExit):
+            exported.setup_register()
+        with pytest.raises(SystemExit):
+            exported.setup_upload()
+
     def test_export_attributes(self, uploadhub, repo, tmpdir):
         checkout = Checkout(uploadhub, repo)
         repo.join("setup.py").write("print 'xyz-1.2.3'")
@@ -148,11 +173,11 @@ class TestUploadFunctional:
 
         # logoff then upload
         devpi("logoff")
-        devpi("upload", "--dry-run", code=404)
+        devpi("upload", "--dry-run")
 
         # go to other index
         devpi("use", "root/pypi")
-        devpi("upload", "--dry-run", code=404)
+        devpi("upload", "--dry-run")
 
     def test_fromdir(self, initproj, devpi, out_devpi, runproc):
         initproj("hello-1.1", {"doc": {
